#define LEDTX_PORT PORTD
#define LEDTX PD5
#define LEDRX_PORT PORTB
#define LEDRX PB2

// board types
#define RESISTIVE 1
#define QPROXTOUCH 2
#define ANADIGIO 3


// input types 
#define ANALOG_INPUT 0
#define DIGITAL_INPUT 1
#define RIBBON_INPUT 2
#define PWM_OUTPUT 3

void midi_handle_sysex(void);

#define MSG_PING 0x01
#define MSG_PONG 0x02
#define MSG_REQSENSETYPE 0x10
#define MSG_SENSETYPE 0x11
#define MSG_SETSENSETYPE 0x12
#define MSG_REQSENSORIOTYPE 0x13
#define MSG_SENSORIOTYPE 0x14
#define MSG_SETSENSORIOTYPE 0x15
#define MSG_REQSENSORSENABLED 0x20
#define MSG_SENSORSENABLED 0x21
#define MSG_SETSENSORSENABLED 0x22
#define MSG_RAWVALUESTART 0x30
#define MSG_RAWVALUE 0x31
#define MSG_RAWVALUESTOP 0x32
#define MSG_REQSENSORCALIB 0x40
#define MSG_SENSORCALIB 0x41
#define MSG_SETSENSORCALIB 0x42
#define MSG_REQSENSORVAR 0x50
#define MSG_SENSORVAR 0x51
#define MSG_SETSENSORVAR 0x52

#define MSG_REQSENSORRATE 0x60
#define MSG_SENSORRATE 0x61
#define MSG_SETSENSORRATE 0x62


#define MSG_DEBUG 0x7F

#if BOARDTYPE == RESISTIVE
#define NUMSENSORS 6
#elif BOARDTYPE == ANADIGIO
#define NUMSENSORS 11
#endif

#define STREAMDATA_FLAG 0x1

// 1 byte per sensor for IO type (input, output, etc)
#define EE_IOTYPE NUMSENSORS
// reserve 8*2bytes locations for up to 8 sensors (just for consistancy
#define EE_SENSORTYPE_ADDR 2*NUMSENSORS
// what sensors are enabled, 2 bytes (up to 16 sensors)
#define EE_SENSORSENABLED (EE_SENSORTYPE_ADDR+2*NUMSENSORS)
// reserve 8*2bytes for calibrated minimums
#define EE_CALIBRATED_MIN (EE_SENSORSENABLED+2)
// reserve 8*2bytes for calibrated middle
#define EE_CALIBRATED_MID (EE_CALIBRATED_MIN+2*NUMSENSORS)
// reserve 8*2bytes for calibrated maximum
#define EE_CALIBRATED_MAX (EE_CALIBRATED_MID+2*NUMSENSORS)
// reserve 8*2 bytes for specific sensortype variables
#define EE_SENSORVARS (EE_CALIBRATED_MAX+2*NUMSENSORS)
// reserve 8 bytes for sensor rate storage
#define EE_SENSORRATE (EE_SENSORVARS+2*NUMSENSORS)

uint16_t read_sensorvars(uint8_t sensenum);
void write_sensorvars(uint8_t sensornum, uint16_t var);

uint16_t read_sensortype(uint8_t sensenum);
void write_sensortype(uint8_t sensenum, uint16_t type);

uint16_t read_sensorsenabled(void);
void write_sensorsenabled(uint16_t d);

uint16_t read_sensorcalib_min(uint8_t sensornum);
uint16_t read_sensorcalib_mid(uint8_t sensornum);
uint16_t read_sensorcalib_max(uint8_t sensornum);
uint16_t read_sensorcalib(uint8_t sensornum, uint8_t eeaddr);
void write_sensorcalib(uint8_t sensornum, uint16_t min, uint16_t mid, uint16_t max);

void write_sensorrate(uint8_t sensenum, uint8_t var);
uint8_t read_sensorrate(uint8_t sensenum);

uint8_t read_sensoriotype(uint8_t sensenum);
void write_sensoriotype(uint8_t sensenum, uint8_t type);

#define INPUT 0
#define OUTPUT 1
void setsensordirection(uint8_t sensor, uint8_t dir);
void enablesensorpullup(uint8_t sensor, uint8_t flag);
void setsensoroutput(uint8_t sensor, uint8_t flag);
void configsensoriotype(uint8_t sensor);
