#include <avr/io.h>
#include <avr/interrupt.h>
#include "util.h"
#include "main.h"

extern volatile uint8_t ledtx_timeout;

void delay_ms(unsigned char ms)
{
  unsigned short delay_count = F_CPU / 4000;
  
  unsigned short cnt;
  asm volatile ("\n"
		"L_dl1%=:\n\t"
		"mov %A0, %A2\n\t"
		"mov %B0, %B2\n"
		"L_dl2%=:\n\t"
		"sbiw %A0, 1\n\t"
		"brne L_dl2%=\n\t"
		"wdr\n\t"
		"dec %1\n\t" "brne L_dl1%=\n\t":"=&w" (cnt)
		:"r"(ms), "r"((unsigned short) (delay_count))
		);
}


uint8_t internal_eeprom_read(uint16_t addr) {
  loop_until_bit_is_clear(EECR, EEWE); // wait for last write to finish
  EEAR = addr;
  EECR |= _BV(EERE);        // start EEPROM read
  return EEDR;            // takes only 1 cycle
}

void internal_eeprom_write(uint16_t addr, uint8_t data) {
  loop_until_bit_is_clear(EECR, EEWE); // wait for last write to finish
  EEAR = addr;
  EEDR = data;
  cli();                // turn off interrupts 
  EECR |= _BV(EEMWE);     // these instructions must happen within 4 cycles
  EECR |= _BV(EEWE);
  sei();                // turn on interrupts again
}
 

void uart_init(uint16_t BRR) {
  /* setup the main UART */


  DDRD |= _BV(1);             // tx output
  DDRD &= ~_BV(0);            // rx input
  UBRRL = BRR;               // set baudrate counter
  UBRRH = 0;

  UCSRA = (1 << U2X);
  UCSRB = (1<<RXEN) | (1<<TXEN) | (1<<RXCIE);    // read and write
  UCSRC = (3<<UCSZ0) | (1 << URSEL) | _BV(USBS);  // 8 bit, 2 stop bits

  // set up interrupt on receive
}

int uart_putchar(char c)
{
  LEDTX_PORT |= _BV(LEDTX);
  ledtx_timeout = 30;   // 30ms
  loop_until_bit_is_set(UCSRA, UDRE);
  UDR = c;
  return 0;
}
